<?php

declare(strict_types=1);

use PrestaShop\Module\LCVPrestaConnector\BridgeAdapter;
use PrestaShop\Module\LCVPrestaConnector\SupplierRefManager;
use PrestaShop\Module\LCVPrestaConnector\SyncConfiguration;
use PrestaShop\PrestaShop\Adapter\SymfonyContainer;

if (!defined('_PS_VERSION_'))
  exit;


class LCVPrestaConnector extends Module
{
    /**
     * Identifiant du module
     */
    public string $Id;

      /**
     * chemin vers tpl
     */
    public string $TplPath;


    /**
     * Préfixe des tables
     */
    public string $TblPrefix;

     /**
     * Configuration du module
     *      
     */
    private ?SyncConfiguration $cfg = null;

    /**
     * Lien vers le pont
     */
    private ?BridgeAdapter $bridge = null;
    

     /**
     * New instance
     * 
     */
    public function __construct()
    {
        // On va se noter une constante pour identifier la version de notre module
        // Cette constante, nommée Id est composée des 3 premieres lettres du nom du module
        $this->Id = substr(basename(__FILE__), 0, 3);
        if ($this->Id == 'pre')
            $this->Id = 'pol';

        $this->TblPrefix = _DB_PREFIX_ . 'pc_' . $this->Id . '_';
        $this->TplPath = '@Modules/prestaconnector/views/templates/';
        
        $this->name = basename(__FILE__, '.php');
        $this->author = 'LCV';
                
        $this->version = '0.0.1';

        $this->tab = 'synchronisation';        
        $this->need_instance = 0;
        $this->ps_versions_compliancy = [
            'min' => '8.0.0',
            'max' => _PS_VERSION_,
        ];
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->trans('LCVPrestaConnector', [], 'Modules.prestaconnector.Admin');
        $this->description = $this->trans('This module provide sync between your Prestashop and your backoffice © LCV.', [], 'Modules.prestaconnector.Admin');

        $this->confirmUninstall = $this->trans('Are you sure you want to uninstall this module ?', [], 'Modules.Polarisprestaconnector.Admin');        

        if (isset($_REQUEST["remount_hook"]))
			$this->registerHooks();
    }

    /**
     * Obtenir les hooks utilisés par ce module
     */
    public function getHooks()
	{
		$hooks = array(
			'dashboardZoneOne',
			'displayBackOfficeHeader',
			);

		return $hooks;
	}

    /**
     * Déclenche une mise à jour de la base si nécessaire !
     */
    public function dbUpgrade()
    {
        // Faut-il déclencher la mise à jour de la base de données du module ?
        // pour le savoir on regarde, si la version enregistrée dans la configuration est inférieure à la version actuelle                
        $currentDbVersion = \Configuration::get($this->Id . 'prestaconnector_version');
        if (!$currentDbVersion)
            $currentDbVersion = '0.0';

        if (version_compare($currentDbVersion, $this->version) < 0) {            
            // On met à jour la version du module
            require_once _PS_MODULE_DIR_ . $this->name . '/db/upgrade.php';
            $upgrade = new \PrestaShop\Module\LCVPrestaConnector\Upgrade();
            $upgrade->upgradeModule($this, $currentDbVersion);
        }
    }
    
    /*
		Réenregistrement des hooks...
	*/
	public function registerHooks($updatePosition = true)
	{
		foreach ($this->getHooks() as $hook)
			$this->registerHook($hook);

		if ($updatePosition)
        {
			$this->updatePosition(Hook::getIdByName('dashboardZoneOne'), 0, 1);
		}
	}

    /**
     * Obtenir l'objet de configuration
     */
    public function getCfg() : SyncConfiguration
    {
        if (!$this->cfg)
            $this->cfg = new SyncConfiguration($this);

        return $this->cfg;
    }

    /**
     * Obtient le pont configuré
     */
    public function getBridge() : BridgeAdapter
    {
        if (!$this->bridge)
        {
            $classname = "PrestaShop\\Module\\LCVPrestaConnector\\" . $this->Id . "\\Bridge";

            $this->bridge = new $classname($this->getCfg());            
        }
        return $this->bridge;
    }

    /*
    * Page de configuration
    */
    public function getContent()
    {
        $this->dbUpgrade();

        Tools::redirectAdmin(
            SymfonyContainer::getInstance()->get('router')->generate('prestaconnector_configuration')
        );
    }

    public function hookDashboardZoneOne($params)
	{
        $router = SymfonyContainer::getInstance()->get('router');
        $tpl_variables = array(
            'module_version' => $this->version,
            'configUrl'=> $router->generate("prestaconnector_configuration"),
            //'statusUrl'=> $router->generate("prestaconnector_status"),
            'logsUrl' => $this->context->link->getAdminLink("AdminLogs")
        );

        return $this->get('twig')->render($this->TplPath . 'hook/dashboard.html.twig', $tpl_variables);
	}


    public function hookDisplayBackOfficeHeader($params)
    {
        $this->context->controller->addJS($this->_path.'views/js/admin.js');
        $this->context->controller->addCSS($this->_path.'views/css/admin.css');

        // Si nous sommes sur l'admin controller, alors on tweak l'interface
        $controller = Tools::getValue('controller');
        if ($controller == "AdminProducts")        
            return $this->tweakAdminProductUI($params);
    }

    /**
     * Contrôle des champs protégés
     * 
     * @param array $params
     *
     * @return string
     */
    public function tweakAdminProductUI($params) : string
    {                
        // Si nous sommes en mode sans synchro de fiche, inutile de protéger les champs, donc on sort
        $cfg = $this->getCfg();
        if ($cfg->get(SyncConfiguration::CFG_WORKMODE) != 1)
            return '';

        $syncCfg = $cfg->get(SyncConfiguration::CFG_DATA_SYNC);
        if (empty($syncCfg))
            return '';

        // Sinon, on va chercher à savoir si le produit est un produit qu'on entretien. 
        // Si nous retrouvons des références fournisseurs pour ce produit, alors c'est le cas
        $id_product = Tools::getValue('id_product');
		if (empty($id_product))
			return '';

        // On va chercher les références fournisseurs
        $refMngr = new SupplierRefManager($this->getCfg()->get(SyncConfiguration::CFG_SUPPLIER), [$id_product]);
        if ($refMngr->getNbProducts() == 0)
            return '';    // Non pas géré

        $class = $this->Id . "-disabled";
        $polarisColor = "#e4a12d";
		$style = "";
		$js = "";

		$js .= "$('.product-type-preview').before('<p class=\"".$class."\" >(Les champs de cette couleur sont contrôlés par votre backoffice ".$this->getBridge()->getId()." et <strong>vous ne devez pas les modifier</strong> directement sous Prestashop)</p>');\n";

        $item_to_disable = [
			'.product-type-preview',        // Pas de changement de type de produit
            '#combination-list-actions',    // ni de création de déclinaisons                        
		];

        $fields_to_disable = [
            '#product_pricing_retail_price_price_tax_included', // Pas d'actions sur les prix
            '#product_pricing_retail_price_price_tax_excluded',            
        ];

        $fields_to_style = [
            '.delete-combination-item',     // pas de suppression de déclinaisons            
            '.edit-combination-item',       // pas de modification de déclinaisons
		];

        // Cas particulier des caractéristiques
        $features = $this->getCfg()->get(SyncConfiguration::CFG_PR_ATTRIBUTES_MAPPING);
        $caracts = [];
        $id_lang = $this->context->language->id;
        if (!empty($features) && isset($features[SyncConfiguration::PRODUCT_ATTR_FEATURE]))
        {            
            foreach ($features[SyncConfiguration::PRODUCT_ATTR_FEATURE] as $feature => $id_feature)
            {
                if ($id_feature > 0)
                {
                    $feature = new Feature($id_feature);
                    $caracts[] = mb_strtolower($feature->name[$id_lang]);
                }
            }
        }
        
        if (!empty($caracts))
            $js .= "$('#product_details_features_feature_values').before('<p class=\"".$class."\">Attention, certaines caractéristiques (".join(", ", $caracts).") sont maintenues par le module ".$this->name.".<br>Ne modifiez pas ces caractéristiques, le module annulera vos modifications pour ces catégories.</p>');\n";


        // Application des données de synchronisation
        foreach( [
            SyncConfiguration::CFG_PRODUCT_SYNC_REF => '#product_details_references_reference',
            SyncConfiguration::CFG_PRODUCT_SYNC_NAME => '#product_header_name',
            SyncConfiguration::CFG_PRODUCT_SYNC_DESCRIPTION => '#product_description_description',
            SyncConfiguration::CFG_PRODUCT_SYNC_MANUFACTURER => '#product_description_manufacturer',
            SyncConfiguration::CFG_PRODUCT_SYNC_DIMENSIONS => ['#product_shipping_dimensions_width', '#product_shipping_dimensions_height', '#product_shipping_dimensions_depth'],
            SyncConfiguration::CFG_PRODUCT_SYNC_WEIGHT => '#product_shipping_dimensions_weight',
            //SyncConfiguration::CFG_PRODUCT_SYNC_ECOTAX => true,
            //SyncConfiguration::CFG_PRODUCT_SYNC_LOCATION => '#combination_form_stock_options_stock_location',
            SyncConfiguration::CFG_PRODUCT_SYNC_TAXRATE => '#product_pricing_retail_price_tax_rules_group_id',
            SyncConfiguration::CFG_PRODUCT_SYNC_EAN13 => '#product_details_references_ean_13',            
            SyncConfiguration::CFG_PRODUCT_SYNC_CATEGORIES => '#product_description_categories',
            SyncConfiguration::CFG_PRODUCT_SYNC_PHOTOS => '#product_description_images',            
        ] as $k => $v)
        {
            if (isset($syncCfg[$k]) && $syncCfg[$k])
            {
                if (is_array($v))
                    $fields_to_disable = array_merge($fields_to_disable, $v);
                else
                    $fields_to_disable[] = $v;
            }
        }

        // Application
        foreach ($item_to_disable as $field)
			$js .= "$('".$field."').attr('tabindex', '-1');\n$('".$field."').addClass('".$class."');\n";

		foreach ($fields_to_disable as $field)
            $js .= "$('".$field."').closest('.form-group').addClass('".$class."');\n"; // Remonter le champs pour mettre la classe sur le premier form-group qui englobe notre classe !            

        // Pour les fields_to_style, on laisse la possibilité tout de même de les modifier, mais on les met en couleur
        foreach ($fields_to_style as $field)
            $style .= $field." { color: $polarisColor !important; border-color: $polarisColor !important;}\n";

        $style .= ".".$class." { pointer-events: none !important; color: $polarisColor !important; border-color: $polarisColor !important;}\n";
        $style .= ".".$class." * { pointer-events: none !important; color: $polarisColor !important; border-color: $polarisColor !important;}\n";
		//$style .= join(", ", $fields_to_hide)." { display: none !important; }\n";

        $output = "";
        if (!empty($style))
			$output .= "<style>\n".$style."</style>\n";
		if (!empty($js))
			$output .= "<script language='javascript'>\n$(document).ready(function() {".$js."});\n</script>\n";

		return $output;
    }
}