<?php
namespace PrestaShop\Module\LCVPrestaConnector;

use PrestaShop\Module\LCVPrestaConnector\Models\Product;

/**
 * Interface de pont
 */
abstract class BridgeAdapter
{

    

    /**
     * Configuration du module
     */
    protected SyncConfiguration $config;

    /**
     * Constructeur
     */
    public function __construct(SyncConfiguration $config )
    {
        $this->config = $config;
    }

    /**
     * Renvoi l'identifiant du pont correspondant au backoffice géré
     */
    public abstract function getId() : string;

    /**
     * Renvoi les spécificités du pont sous forme de tableau associatif clé/valeur de type string
     * 
     * @return array<string,string>
     */
    public abstract function getSpecifics() : array;

    /**
     * Teste la connexion au backoffice avec les paramètres fournis.
     * Renvoi null quand la connexion a été effectuée correctement,
     * autrement, renvoi l'erreur.
     */
    public abstract function bind($args) : string|null;

    /**
     * Configuration par défaut de ce pont
     * 
     */
    public abstract function alterDefaultCfg(array & $cfg);

    /**
     * Révoque les configurations non supportées par le pont
     * 
     * @param array $cfg Configuration à révoquer
     */
    public abstract function revokeUnsupportedCfg(array & $cfg);

    /**
     * Obtient les produits à synchroniser sous forme de tableau
     * 
     * @return array<Product>
     */
    public abstract function & nextProducts(): array;

    /**
     * Obtient les stocks à synchroniser sous forme de tableau
     * 
     * @return array<Stock>
     */
    public abstract function & nextStocks(): array;

    /**
     * Importe la configuration du backoffice via les API
     */
    public abstract function pullEnvironment();

    /**
     * Obtient l'URL d'une photo par sa clé
     */
    public abstract function getPhotoURL(string $photo): string;

    /**
     * Réinitialise la synchronisation
     * (synchro totale)
     */
    public function resetSyncPosition()
    {
        $empty = [];
        $this->config->set(SyncConfiguration::CFG_SYNC_POSITIONS, $empty);
    }

    /**
     * Obtient la position de synchronisation pour un type de données précisé
     * 
     * @param string $type Type de données
     * @return mixed Position de synchronisation
     */
    protected function getSyncPosition(string $type) : mixed
    {
        $pos = $this->config->get(SyncConfiguration::CFG_SYNC_POSITIONS);
        if (!isset($pos))
            $pos = [];
        if (!isset($pos[$type]))
            return null;
        return $pos[$type];
    }

    /**
     * Obtient la position de synchronisation pour un type de données précisé
     * 
     * @param string $type Type de données
     * @return mixed Position de synchronisation
     */
    protected function setSyncPosition(string $type, mixed $position)
    {
        $pos = $this->config->get(SyncConfiguration::CFG_SYNC_POSITIONS);
        if (!isset($pos))
            $pos = [];
        $pos[$type] = $position;        
        $pos = $this->config->set(SyncConfiguration::CFG_SYNC_POSITIONS, $pos);
    }
}
