<?php
namespace PrestaShop\Module\LCVPrestaConnector;

/**
 * Gestionnaire de référence fournisseur
 * 
 */
class SupplierRefManager
{
    /**
     * Identifiant du fournisseur
     */
    private string $supplierId;

    /**
     * Cache des références fournisseur pour les produits
     * 
     * @var array<string, int> tableau associatif de références fournisseur => id de produit
     */
    private array $productsCache = [];    

    /**
     * Cache des références fournisseur pour les déclinaisons
     * 
     * @var array<string, int> tableau associatif de références fournisseur => id de combinaisons (productsAttribute)
     */
    private array $combinationsCache = [];

    /** 
     * Instancie un nouveau gestionnaire de référence
     * 
     * @param int $supplierId identifiant du fournisseur
     * @param array<int> $productsId produits gérés, ou aucun pour tout charger
    */
    public function __construct($supplierId, ?array $productsId = null)    
    {
        $this->supplierId = $supplierId;

        $db = \Db::getInstance();

        $this->productsCache = [];
        $this->combinationsCache = [];        

        // Chargement des références fournisseur depuis le fichier product_supplier
        $sql = 'SELECT `id_product`, `id_product_attribute`, `product_supplier_reference` FROM `'._DB_PREFIX_.'product_supplier` WHERE `id_supplier` = '.$supplierId;
        if (isset ($productsId) && !empty($productsId))
            $sql .= ' AND `id_product` IN ('.implode(',', $productsId).')';
        $res = $db->executeS($sql);
        if ($res === FALSE)
            throw new \Exception('Erreur lors de la récupération des références fournisseur : '.$db->getMsgError());

        foreach ($res as $row)
        {
            $this->combinationsCache[$row['product_supplier_reference']] = (int) $row['id_product_attribute'];            
            $this->productsCache[$row['product_supplier_reference']] = (int) $row['id_product'];            
        }
    }

    /**
     * Renvoi le nombre de références produit dans le cache
     */
    public function getNbProducts() : int
    {
        return count($this->productsCache);
    }

    /**
     * Traduit une référence fournisseur en identifiant de produit
     * 
     * @param string $ref référence fournisseur à traduire
     * @return int|null identifiant de produit, ou null si non trouvé
     */
    public function translateProductRef(string $ref) : ?int
    {
        return $this->productsCache[$ref] ?? null;
    }

    /**
     * Traduit une référence fournisseur en identifiant de combinaison
     * 
     * @param string $ref référence fournisseur à traduire
     * @return int|null identifiant de combinaison, ou null si non trouvé
     */
    public function translateCombinationRef(string $ref) : ?int
    {
        return $this->combinationsCache[$ref] ?? null;
    }    

    /**
     * Enregistre une référence fournisseur pour un produit
     * 
     * @param string $ref référence fournisseur à enregistrer
     * @param int $id_product identifiant du produit
     */
    public function registerProductRef(string $ref, int $id_product)
    {
        // Obtenir la devise par défaut du système
        $currencyId = \Configuration::get('PS_CURRENCY_DEFAULT');

        $db = \Db::getInstance();
        
        // On remplace les données dans la base de données, si elles existent déjà, ou on les insère
        // (REPLACE)
        $sql = 'REPLACE INTO `'._DB_PREFIX_.'product_supplier` (`id_product`, `id_product_attribute`, `id_supplier`, `id_currency`, `product_supplier_reference`) VALUES ('.$id_product.', 0, '.$this->supplierId.',  '.$currencyId.', 
                    \''.$db->_escape($ref).'\')';
        if ($db->execute($sql) === false)
            throw new \Exception('Erreur lors de l\'enregistrement de la référence fournisseur '.$ref.' : '.$db->getMsgError());

        $this->productsCache[$ref] = $id_product;
    }

    /**
     * Enregistre une référence fournisseur pour un produit
     * 
     * @param string $ref référence fournisseur à enregistrer
     * @param int $id_product identifiant du produit
     * @param int $id_product_attribute identifiant de la déclinaison
     */
    public function registerCombinationRef(string $ref, int $id_product, int $id_product_attribute)
    {
        // Obtenir la devise par défaut du système
        $currencyId = \Configuration::get('PS_CURRENCY_DEFAULT');

        $db = \Db::getInstance();
        
        // On remplace les données dans la base de données, si elles existent déjà, ou on les insère
        // (REPLACE)
        $sql = 'REPLACE INTO `'._DB_PREFIX_.'product_supplier` (`id_product`, `id_product_attribute`, `id_supplier`, `id_currency`, `product_supplier_reference`) VALUES ('.$id_product.', '. $id_product_attribute .', '.$this->supplierId.',  '.$currencyId.', 
                    \''.$db->_escape($ref).'\')';
        if ($db->execute($sql) === false)
            throw new \Exception('Erreur lors de l\'enregistrement de la référence fournisseur '.$ref.' : '.$db->getMsgError());

        $this->combinationsCache[$ref] = $id_product_attribute;
    }

    /**
     * Efface une référence fournisseur produit car elle n'est plus valide
     * 
     */
    public function cleanProductRef(string $ref)
    {
        $db = \Db::getInstance();
        
        // On efface les données dans la base de données
        $sql = 'DELETE FROM `'._DB_PREFIX_.'product_supplier` WHERE `id_supplier` = '.$this->supplierId.' AND `id_product_attribute` = 0 AND `product_supplier_reference` = \''.$db->_escape($ref).'\'';
        if ($db->execute($sql) === false)
            throw new \Exception('Erreur lors de l\'effacement de la référence fournisseur '.$ref.' : '.$db->getMsgError());

        unset($this->productsCache[$ref]);
    }

    /**
     * Efface une référence fournisseur déclinaison car elle n'est plus valide
     */
    public function cleanCombinationRef(string $ref)
    {
        $db = \Db::getInstance();
        
        // On efface les données dans la base de données
        $sql = 'DELETE FROM `'._DB_PREFIX_.'product_supplier` WHERE `id_supplier` = '.$this->supplierId.' AND `id_product_attribute` != 0 AND `product_supplier_reference` = \''.$db->_escape($ref).'\'';
        if ($db->execute($sql) === false)
            throw new \Exception('Erreur lors de l\'effacement de la référence fournisseur '.$ref.' : '.$db->getMsgError());

        unset($this->combinationsCache[$ref]);
    }
}