<?php
namespace PrestaShop\Module\LCVPrestaConnector\lcv;

use PrestaShop\Module\LCVPrestaConnector\BridgeAdapter;
use PrestaShop\Module\LCVPrestaConnector\Models\Article;
use PrestaShop\Module\LCVPrestaConnector\Models\AttributeRef;
use PrestaShop\Module\LCVPrestaConnector\Models\Product;
use PrestaShop\Module\LCVPrestaConnector\SyncConfiguration;

/**
 * Pont vers LCV
 */
class Bridge extends BridgeAdapter
{
    /**
     * Nombre de produits par page
     */
    private const productsPerPage = 100;

    /**
     * Dernière position de produits synchronisés
     */
    private string|null $lastProductPosition = null;

    /**
     * Constructeur
     */
    public function __construct(SyncConfiguration $config)
    {
        parent::__construct($config);        
    }

    /**
     * Renvoi l'identifiant du pont correspondant au backoffice géré
     */
    public function getId() : string
    {
        return 'LCV';
    }

    /**
     * Configuration par défaut de ce pont
     * 
     */
    public function alterDefaultCfg(array & $cfg)
    {
        // A priori, les options par défaut nous convienne, sauf celle non supportées !
        $cfg[SyncConfiguration::CFG_PRODUCT_SYNC_PHOTOS] = false;
        $cfg[SyncConfiguration::CFG_PRODUCT_SYNC_LOCATION] = false;
    }

    /**
     * Révoque les configurations non supportées par le pont
     * 
     * @param array $cfg Configuration à révoquer
     */
    public function revokeUnsupportedCfg(array & $cfg)
    {
        // Certaines fonctions ne sont pas encore disponible pour ce pont
        unset($cfg[SyncConfiguration::CFG_PRODUCT_SYNC_DIMENSIONS]);        // Dimensions produit        
        unset($cfg[SyncConfiguration::CFG_PRODUCT_SYNC_ECOTAX]);            // L'écotaxe
        unset($cfg[SyncConfiguration::CFG_PRODUCT_SYNC_PHOTOS]);            // Photos produit
        unset($cfg[SyncConfiguration::CFG_PRODUCT_SYNC_EAN13]);             // EAN
        unset($cfg[SyncConfiguration::CFG_PRODUCT_SYNC_LOCATION]);          // Zonage                
        unset($cfg[SyncConfiguration::CFG_PRODUCT_SYNC_TAXRATE]);           // Taux de TVA
        unset($cfg[SyncConfiguration::CFG_PRODUCT_SYNC_WEIGHT]);            // Poids
    }

    /**
     * Renvoi les spécificités du pont sous forme de tableau associatif clé/valeur de type string
     * 
     * @return array<string,string>
     */
    public function getSpecifics() : array
    {
        $cfgLogin = $this->config->get(SyncConfiguration::CFG_LOGIN);

        return [
            "ADDRESS_LABEL" => "Quelle est l'adresse de connexion à votre backoffice LCV (il s'agit en général d'une adresse de la forme https://84.66.142.12:8443/)",
            "PREVIOUS_ADDRESS" => isset($cfgLogin) && isset($cfgLogin["address"]) ? $cfgLogin["address"] : "",
        ];
    }

    /**
     * Teste la connexion au backoffice avec les paramètres fournis
     */
    public function bind($args) : string|null
    {
        // Interressons nous à l'URL
        if (!isset($args["address"]) || empty(trim($args["address"])))
            return "1 : l'URL de connexion au service LCV n'est pas définie, ni valide !";

        $url = $args["address"];

        // Si l'adresse ne commence pas par http:// ou https://, on considère que c'est un raccourci, on l'ajoute
        if (substr($url, 0, 5) !== 'http:' && substr($url, 0, 6) != 'https:')
            $url = "https://" . $url;

        // Si nous sommes en production et que l'url commence par http:// alors il s'agit d'un système non sécurisé
        // et nous refusons la connexion
        // on peut savoir si nous sommes en production si le host d'appel est différent de localhost
        if ($_SERVER['HTTP_HOST'] !== 'localhost' && substr($url, 0, 7) === 'http://')
            return "2 : le backoffice LCV ne peut pas être contacté en HTTP, il doit être en HTTPS";

        // On calcule la date du lendemain
        $tomorrow = date('Ymd', strtotime('+1 day'));

        // Testons une connexion
        $apiUrl = sprintf('%s/LectureDesModelesAvecPrix/%s/%s/0/%s/0/0',
                $url,
                $args['username'],   // sUser
                $args['password'],    // sMdp
                urlencode($tomorrow)    // date actuelle
            );

        // Initialisation de la session cURL
        $ch = curl_init();
    
        // Configuration des options cURL pour une requête GET
        curl_setopt($ch, CURLOPT_URL, $apiUrl);
        curl_setopt($ch, CURLOPT_HTTPGET, true); // Utiliser la méthode GET
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); // Pour retourner la réponse comme une chaîne

        curl_setopt($ch, CURLOPT_CERTINFO, true); // Pour récupérer le fingerprint au passage
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);    // NE PAS vérifier la validité du certificat (nom)
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Vérifier que le certificat appartient bien à l'host qu'il désigne
        
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); // Permet de suivre les redirections
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);
          
        // Exécution de la requête cURL
        $response = curl_exec($ch);
        $result = curl_getinfo($ch);

        // Si la vérification du certificat a échouée et que nous n'avons pas de fingerprint passé dans les arguments
        // alors, c'est une erreur
        $pubkeycheck = null;
        if ($result["ssl_verify_result"] != 0)
        {
            // Calcul du fingerprint de la connexion
            $certInfo = !empty($result["certinfo"]) ? $result["certinfo"][0] : null;
            if (!$certInfo)
                return "13 : impossible de récupérer les informations du certificat SSL";        

            // On calcule l'empreinte de la clé publique du certificat
            $fingerprint = openssl_x509_fingerprint($certInfo["Cert"], "sha256");            
            
            if (!isset($args["fingerprint"]))
                return "14 : impossible de vérifier le certificat SSL : Empreinte : " . $fingerprint;

            if (empty(trim($args["fingerprint"])))
                return "16 : il semble que votre backoffice ne dispose pas d'un certificat SSL vérifié par un tier. Vérifiez l'empreinte du certificat SSL. Empreinte : " . $fingerprint;

            if ($args["fingerprint"] != $fingerprint)
                return "15 : le certificat SSL ne correspond pas à l'empreinte fournie. Empreinte : " . $fingerprint;
            // On extrait la clé publique du certificat au format binaire pour en faire un hash256
            // on isole le DER de la clé publique
            $pemPubKey = openssl_pkey_get_details(openssl_pkey_get_public($certInfo["Cert"]));
            $pemPubKey = $pemPubKey["key"];
                
            $begin = "KEY-----";
            $end   = "-----END";
            $pemPubKey = substr($pemPubKey, strpos($pemPubKey, $begin)+strlen($begin));   
            $pemPubKey = substr($pemPubKey, 0, strpos($pemPubKey, $end));

            $der = base64_decode($pemPubKey);

            // Etape 2 : on préviens CURL de ne répondre qu'à cette clé publique
            $pubkeycheck = "sha256//".base64_encode(hash("sha256", $der, true));
        }
        
        if ($result["http_code"] == 0)
            return "6 : impossible de se connecter au serveur";

        // Si nous avons une erreur 404, c'est très certainement car le serveur n'est pas un backoffice LCV
        if ($result["http_code"] == 404)
            return "7 : le serveur ne semble pas être un backoffice LCV";

        // Erreur divers avec le serveur
       // if ($result["http_code"] != 200)
        //    return "8 : erreur lors de la connexion au service LCV : " . $result["http_code"];

        // Fermer cURL
        curl_close($ch);

        // On vérifie maintenant que le nom d'utilisateur et le mot de passe sont corrects
        // Ils sont dans le paramètre $args sous les termes "username" et "password"
        if (!isset($args["username"]) || empty(trim($args["username"])))
            return "20 : vous devez saisir des identifiants valides pour vous connecter à votre service LCV"; 

        if (!isset($args["password"]) || empty(trim($args["password"])))
            return "21 : vous n'avez pas précisé votre mot de passe !";

        // Testons maintenant une connexion AVEC les identifiants et un pinning du certificat
        $apiUrl = sprintf('%s/LectureDesModelesAvecPrix/%s/%s/0/%s/0/0',
                $url,
                urlencode($args['username']),   // sUser
                urlencode($args['password']),    // sMdp
                urlencode($tomorrow)    // date actuelle
            );

        // Initialisation de la session cURL
        $ch = curl_init();
            
        // Configuration des options cURL pour une requête GET
        curl_setopt($ch, CURLOPT_URL, $apiUrl);
        curl_setopt($ch, CURLOPT_HTTPGET, true); // Utiliser la méthode GET
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); // Pour retourner la réponse comme une chaîne

        curl_setopt($ch, CURLOPT_CERTINFO, true); // Pour récupérer le fingerprint au passage
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);    // NE PAS vérifier la validité du certificat (nom)
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Vérifier que le certificat appartient bien à l'host qu'il désigne

        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); // Permet de suivre les redirections
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);        

        // Pin le certificat sur l'empreinte passée en paramètre si elle existe
        if ($pubkeycheck)
            curl_setopt($ch, CURLOPT_PINNEDPUBLICKEY, $pubkeycheck);
        
        // Exécution de la requête cURL
        $response = curl_exec($ch);
        $result = curl_getinfo($ch);        
        
        if ($result["http_code"] != 200)
            return "22 : erreur lors de la vérification des identifiants LCV : " . $result["http_code"];

        // On regarde si nous n'avons pas le message de retour dans la réponse qui 
        // ressemble à { "Resultat":"Erreur... Utilisateur inconnu" }
        // si c'est le cas, on renvoi le message d'erreur
        $json = json_decode($response);
        if (isset($json->Resultat) && str_contains($json->Resultat, "Erreur"))
            return "23 : ".$json->Resultat;

        // sauver la config
        $loginCfg = [
            'address' => $url,
            'username' => $args["username"],
            'password' => $args["password"],
            'fingerprint' => $args["fingerprint"],
            'pubkeycheck' => $pubkeycheck,
        ];
        $this->config->set(SyncConfiguration::CFG_LOGIN, $loginCfg);

        // Et dire que tout s'est bien passé
        return null;
    }

    /**
     * Obtient l'URL d'une photo par sa clé
     */
    public function getPhotoURL(string $photo) : string
    {
        throw new \Exception('Not implemented');
    }

    /**
     * Tirage des données par API
     * 
     * @param string $api API à appeler
     * @param array|null $args Arguments à passer à l'API
     */
    private function getByApi(string $api, ?array $args = null) : array
    {
        $cfgLogin = $this->config->get(SyncConfiguration::CFG_LOGIN);
        if (!$cfgLogin)
            throw new \Exception('Erreur de configuration : le module n\'est pas associé au backoffice LCV !');

        // Initialisation de la session cURL
        $ch = curl_init();

        $apiUrl = sprintf('%s/%s/%s/%s/0',
                $cfgLogin["address"],
                urlencode($api),
                urlencode($cfgLogin["username"]),
                urlencode($cfgLogin["password"])
            );

        if ($args)
            foreach ($args as $v)
                $apiUrl .= '/' . urlencode($v);

        // Configuration des options cURL pour une requête GET
        curl_setopt($ch, CURLOPT_URL, $apiUrl);
        curl_setopt($ch, CURLOPT_HTTPGET, true); // Utiliser la méthode GET
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); // Pour retourner la réponse comme une chaîne

        curl_setopt($ch, CURLOPT_CERTINFO, true); // Pour récupérer le fingerprint au passage
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);    // NE PAS vérifier la validité du certificat (nom)
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Vérifier que le certificat appartient bien à l'host qu'il désigne

        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); // Permet de suivre les redirections
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);        

        // Pin le certificat sur l'empreinte passée en paramètre si elle existe
        if (isset($cfgLogin["pubkeycheck"]) && !empty($cfgLogin["pubkeycheck"]))
            curl_setopt($ch, CURLOPT_PINNEDPUBLICKEY, $cfgLogin["pubkeycheck"]);

        // Envoi de la requête
        $response = curl_exec($ch);
        $result = curl_getinfo($ch);

        // S'il y a une erreur, on renvoi une exception
        if ($result["http_code"] < 200 || $result["http_code"] >= 300)
            throw new \Exception("Erreur lors de la récupération des données : " . $result["http_code"] . " : " . $response);

        // Fermer cURL
        curl_close($ch);

        // On retourne le résultat
        return json_decode($response, true);
    }

    /**
     * Importe la configuration du backoffice via les API
     */
    public function pullEnvironment()
    {
        // Tirage
        $infos = $this->getByApi('LectureDesTables'); // Tirage des modèles
        
        // Tirage paramètres généraux

        // Fixation du magasin web
        $magweb = isset($infos["Table_des_Parametres_Generaux"]["Parametres"]["MagWeb"]) ? $infos["Table_des_Parametres_Generaux"]["Parametres"]["MagWeb"] : null;
        if ($this->config->get(SyncConfiguration::CFG_WEB_STORE_CODE) !== $magweb)
            $this->config->set(SyncConfiguration::CFG_WEB_STORE_CODE, $magweb);

        // Tirage des grilles de taille
        foreach ($infos["Table_des_Grilles_de_Taille"]["Grillesdetaille"] as $grTailles)
        {
            if ($this->config->getMap(SyncConfiguration::CFG_MAP_SIZEGRIDS, $grTailles["CodeGrille"]) === null)
                $this->config->setMap(SyncConfiguration::CFG_MAP_SIZEGRIDS, $grTailles["CodeGrille"], 0, true);

            $name = $grTailles["CodeGrille"] . ' - '. $grTailles["NomGrille"];

            if ($this->config->getMap(SyncConfiguration::CFG_MAP_SIZEGRIDS_NAME, $grTailles["CodeGrille"]) != $name)
                $this->config->setMap(SyncConfiguration::CFG_MAP_SIZEGRIDS_NAME, $grTailles["CodeGrille"], $name);
        }

        // Tirage des marques, à défaut les fournisseurs
        $tblMarques = null;

        // Tirage des catégories dynamiques
        $empty = [ "*" => 0 ];
        $dyncats = [];
        $dynattrs = [];
        
        foreach ($infos["Table_Article"]["IdTable"] as $cat)
        {
            $nomTable = mb_strtolower($cat["NomTable"]);
            $dynattrs[] = $nomTable;

            /*// Importation !
            Importation des attributs si jamais on a pas le choix
            $map = SyncConfiguration::CFG_MAP_DYN_CAT . $nomTable;
            if ($this->config->get($map) === null)  // auto création
                $this->config->set($map, $empty);

            $mapNames = SyncConfiguration::CFG_MAP_DYN_CAT . $nomTable."_name";
            if ($this->config->get($mapNames) === null)  // auto création
                $this->config->set($mapNames, $empty);

            foreach ($cat["Description"]["TableArti"] as $category)
            {
                $code = $category["Code"];
                if (!$code) continue; // possibilité de null

                $name = $category["Code"] . ' - ' . $category["Libelle"];
                if ($this->config->getMap($map, $code) === null)
                    $this->config->setMap($map, $code, 0, true);

                if ($this->config->getMap($mapNames, $code) != $name)
                    $this->config->setMap($mapNames, $code, $name);
            }*/
        }

        foreach ($infos["Table_Modele"]["IdTable"] as $cat)
        {
            $nomTable = mb_strtolower($cat["NomTable"]);
            switch ($nomTable)
            {
                case "marque":
                    $tblMarques = $cat["Description"]["TableMdle"];
                    break;
                default:
                    // Importation !
                    $map = SyncConfiguration::CFG_MAP_DYN_CAT . $nomTable;
                    if ($this->config->get($map) === null)  /* auto création */
                        $this->config->set($map, $empty);

                    $dyncats[] = $nomTable;

                    $mapNames = SyncConfiguration::CFG_MAP_DYN_CAT . $nomTable."_name";
                    if ($this->config->get($mapNames) === null)  /* auto création */
                        $this->config->set($mapNames, $empty);

                    foreach ($cat["Description"]["TableMdle"] as $category)
                    {
                        $code = $category["Code"];
                        if (!$code) continue; // possibilité de null

                        $name = $category["Code"] . ' - ' . $category["Libelle"];
                        if ($this->config->getMap($map, $code) === null)
                            $this->config->setMap($map, $code, 0, true);

                        if ($this->config->getMap($mapNames, $code) != $name)
                            $this->config->setMap($mapNames, $code, $name);
                    }
                    break;
            }
        }

        // Saisons
        $nomTable = "saison";            
        $map = SyncConfiguration::CFG_MAP_DYN_CAT . $nomTable;
        if ($this->config->get($map) === null)  /* auto création */
            $this->config->set($map, $empty);

        $dyncats[] = $nomTable;

        $mapNames = SyncConfiguration::CFG_MAP_DYN_CAT . $nomTable."_name";
        if ($this->config->get($mapNames) === null)  /* auto création */
            $this->config->set($mapNames, $empty);

        foreach ($infos["Table_des_Saisons"]["Saison"] as $category)
        {
            $code = $category["Code"];
            if (!$code) continue; // possibilité de null

            $name = $category["Code"] . ' - ' . $category["Libelle"];
            if ($this->config->getMap($map, $code) === null)
                $this->config->setMap($map, $code, 0, true);

            if ($this->config->getMap($mapNames, $code) != $name)
                $this->config->setMap($mapNames, $code, $name);
        }

        if (!$tblMarques)   // Pas de marque, on prend les fournisseurs !
            $tblMarques = $infos["Table_des_Fournisseurs"]["Fournisseur"];

        // Catégories dynamiques
        $this->config->set(SyncConfiguration::CFG_DYN_CAT, $dyncats);
        $this->config->set(SyncConfiguration::CFG_DYN_ATTR, $dynattrs);

        if ($this->config->get(SyncConfiguration::CFG_MAP_MANUFACTURER) === null)
            $this->config->set(SyncConfiguration::CFG_MAP_MANUFACTURER, $empty);

        if ($tblMarques)
        {
            foreach ($tblMarques as $marque)
            {
                if (!$marque["Code"]) continue; // possibilité de null
                if ($this->config->getMap(SyncConfiguration::CFG_MAP_MANUFACTURER, $marque["Code"]) === null)
                    $this->config->setMap(SyncConfiguration::CFG_MAP_MANUFACTURER, $marque["Code"], 0, true);

                $name = $marque["Code"] . ' - '. $marque["Libelle"];

                if ($this->config->getMap(SyncConfiguration::CFG_MAP_MANUFACTURER_NAME, $marque["Code"]) != $name)
                    $this->config->setMap(SyncConfiguration::CFG_MAP_MANUFACTURER_NAME, $marque["Code"], $name);
            }
        }
    }

    /**
     * Obtient les produits à synchroniser sous forme de tableau
     * 
     * @return array<Product>
     */
    public function & nextProducts() : array
    {
        $products = [];
        // Si nous avons une ancienne position à sauver, c'est le moment
        if ($this->lastProductPosition)
        {            
            $this->setSyncPosition('products', $this->lastProductPosition);
            $this->lastProductPosition = null;
        }

        $codeMagasin = $this->config->get(SyncConfiguration::CFG_WEB_STORE_CODE);
        if (!$codeMagasin)
        {
            $this->pullEnvironment();   
            $codeMagasin = $this->config->get(SyncConfiguration::CFG_WEB_STORE_CODE);
            if (!$codeMagasin)
                throw new \Exception('Erreur de configuration : le magasin web n\'est pas défini');
        }

        $position = explode('/', $this->getSyncPosition('products') ?? '19000101/0');
        if (count($position) < 2)
            $position[1] = 0;

        // Tirage des produits
        foreach ($this->getByApi('LectureDesModelesAvecPrix', [ $position[0], $position[1], self::productsPerPage ])["Modele"] as $source)
        {
            if (isset($source["Article"]["NumArti"]))
                $declinaisons = [ $source["Article"] ]; // Un seul
            else
                $declinaisons = $source["Article"]; // J'ai un tableau 
            
            foreach ($declinaisons as $declinaison)
            {
                // Construction du produit
                $product = new Product($source["NumMdle"], $source["CodMdle"]);

                // Description produit
                if (isset($declinaison["Desart"]))
                    $product->description = $declinaison["Desart"];

                // Attributs du produit
                $product->productAttributes = [];

                foreach ($this->config->get(SyncConfiguration::CFG_DYN_CAT) as $dynCat)
                    if (isset($source[$dynCat]))
                        $product->productAttributes[$dynCat] = new AttributeRef($source[$dynCat], $source[$dynCat]);

                foreach ($this->config->get(SyncConfiguration::CFG_DYN_ATTR) as $attrCat)
                    if (isset($declinaison[$attrCat]))
                        $product->productAttributes[$dynCat] = new AttributeRef($declinaison[$attrCat], $declinaison[$attrCat]);

                // Grille de tailles
                $product->grilleTailles = (string) $source["Grille"];

                // Taxe
                $product->taxrate = 0.20; // ?

                // Les dimensions / poids
                //$product->weight = $source->getPoids();            

                // Les déclinaisons (tailles)
                if (isset($declinaison["TarifMagInternet"]["QtTaille"]))
                {
                    if (isset($declinaison["TarifMagInternet"]["QtTaille"]["Indice"]))
                        $tailles = [ $declinaison["TarifMagInternet"]["QtTaille"] ];    // Une seule taille ?
                    else
                        $tailles = $declinaison["TarifMagInternet"]["QtTaille"];    // Un tableau de taille !

                    foreach ($tailles as $taille)
                    {
                        $refs = $taille["CodeBarre"];
                        if (empty($refs) || empty($taille["Taille"])) continue;

                        $article = new Article($refs, $taille["Taille"]);

                        // Prix TTC
                        $article->priceTTC = $taille["PrixVente"];

                        $product->articles[] = $article;
                    }
                }
                
                // Nom du produit
                $product->name = $source["Libelle"];

                // Marque
                if (isset($source["Marque"]))
                    $product->manufacturer = new AttributeRef($source["Marque"], $source["Marque"]);
                else
                    $product->manufacturer = new AttributeRef($source["CodFour"], $source["NomFour"]);
                
                $products[] = $product;
            }
        }

        // Fini
        if (count($products) == 0)
        {
            $position[0] = date('Ymd');    // a priori, on a tout tiré, on déplace notre date de modification pour accélérer la prochaine fois
            $position[1] = 0;   // Et donc du coup, ça sera la première page
            $this->lastProductPosition = $position[0] . '/' . $position[1];
            $this->setSyncPosition('products', $this->lastProductPosition);
        }
        else
            $position[1]++; // page suivante

        $this->lastProductPosition = $position[0] . '/' . $position[1];

        return $products;
    }

    /**
     * Obtient les stocks à synchroniser sous forme de tableau
     * 
     * @return array<Stock>
     */
    public function & nextStocks(): array
    {
        throw new \Exception('Non implémenté');
    }
}
